import 'device.m.dart';
import 'aIDiagnosis.m.dart';

import 'package:fis_jsonrpc/utils.dart';

enum LogEventLevel {
	Verbose,
	Debug,
	Information,
	Warning,
	Error,
	Fatal,
}

class DBLog {
	LogEventLevel level;
	String? utcTimestamp;
	String? messageTemplate;
	DateTime? timestamp;

	DBLog({
		this.level = LogEventLevel.Verbose,
		this.utcTimestamp,
		this.messageTemplate,
		this.timestamp,
	});

	factory DBLog.fromJson(Map<String, dynamic> map) {
		return DBLog( 
			level: LogEventLevel.values.firstWhere((e) => e.index == map['Level']),
			utcTimestamp: map['UtcTimestamp'],
			messageTemplate: map['MessageTemplate'],
			timestamp: map['Timestamp'] != null ? DateTime.parse(map['Timestamp']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['Level'] = level.index;
		if(utcTimestamp != null)
			map['UtcTimestamp'] = utcTimestamp;
		if(messageTemplate != null)
			map['MessageTemplate'] = messageTemplate;
		if(timestamp != null)
			map['Timestamp'] = JsonRpcUtils.dateFormat(timestamp!);
		return map;
	}
}

class GetDBLogPagesRequest extends PageFilterRequest{
	String? level;
	String? keyWord;
	String? exclude;
	DateTime? startTime;
	DateTime? endTime;

	GetDBLogPagesRequest({
		this.level,
		this.keyWord,
		this.exclude,
		this.startTime,
		this.endTime,
		int currentPage = 0,
		int pageSize = 0,
		Map<String,String>? filter,
		bool isFuzzy = false,
	}) : super(
			currentPage: currentPage,
			pageSize: pageSize,
			filter: filter,
			isFuzzy: isFuzzy,
		);

	factory GetDBLogPagesRequest.fromJson(Map<String, dynamic> map) {
		return GetDBLogPagesRequest( 
			level: map['Level'],
			keyWord: map['KeyWord'],
			exclude: map['Exclude'],
			startTime: map['StartTime'] != null ? DateTime.parse(map['StartTime']) : null,
			endTime: map['EndTime'] != null ? DateTime.parse(map['EndTime']) : null,
			currentPage: map['CurrentPage'],
			pageSize: map['PageSize'],
			filter: map['Filter'] != null ? map['Filter'].cast<String,String>() : null,
			isFuzzy: map['IsFuzzy'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(level != null)
			map['Level'] = level;
		if(keyWord != null)
			map['KeyWord'] = keyWord;
		if(exclude != null)
			map['Exclude'] = exclude;
		if(startTime != null)
			map['StartTime'] = JsonRpcUtils.dateFormat(startTime!);
		if(endTime != null)
			map['EndTime'] = JsonRpcUtils.dateFormat(endTime!);
		return map;
	}
}

class DeleteDBLogsRequest extends PageFilterRequest{
	int daysAgo;

	DeleteDBLogsRequest({
		this.daysAgo = 0,
		int currentPage = 0,
		int pageSize = 0,
		Map<String,String>? filter,
		bool isFuzzy = false,
	}) : super(
			currentPage: currentPage,
			pageSize: pageSize,
			filter: filter,
			isFuzzy: isFuzzy,
		);

	factory DeleteDBLogsRequest.fromJson(Map<String, dynamic> map) {
		return DeleteDBLogsRequest( 
			daysAgo: map['DaysAgo'],
			currentPage: map['CurrentPage'],
			pageSize: map['PageSize'],
			filter: map['Filter'] != null ? map['Filter'].cast<String,String>() : null,
			isFuzzy: map['IsFuzzy'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['DaysAgo'] = daysAgo;
		return map;
	}
}


