import 'connect.m.dart';
import 'organization.m.dart';
import 'aIDiagnosis.m.dart';
import 'patient.m.dart';
import 'authentication.m.dart';

import 'package:fis_jsonrpc/utils.dart';

class FindHigherOrganizationsRequest extends TokenRequest{
	String? organizationName;

	FindHigherOrganizationsRequest({
		this.organizationName,
		String? token,
	}) : super(
			token: token,
		);

	factory FindHigherOrganizationsRequest.fromJson(Map<String, dynamic> map) {
		return FindHigherOrganizationsRequest( 
			organizationName: map['OrganizationName'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(organizationName != null)
			map['OrganizationName'] = organizationName;
		return map;
	}
}

class FindGrassRootsOrganizationsRequest extends TokenRequest{
	String? organizationName;

	FindGrassRootsOrganizationsRequest({
		this.organizationName,
		String? token,
	}) : super(
			token: token,
		);

	factory FindGrassRootsOrganizationsRequest.fromJson(Map<String, dynamic> map) {
		return FindGrassRootsOrganizationsRequest( 
			organizationName: map['OrganizationName'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(organizationName != null)
			map['OrganizationName'] = organizationName;
		return map;
	}
}

class OrganizationExpertDTO extends UserBaseDTO{
	String? fullName;
	List<String >? fieldList;

	OrganizationExpertDTO({
		this.fullName,
		this.fieldList,
		String? userCode,
		String? userName,
		String? headImageUrl,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			userCode: userCode,
			userName: userName,
			headImageUrl: headImageUrl,
			createTime: createTime,
			updateTime: updateTime,
		);

	factory OrganizationExpertDTO.fromJson(Map<String, dynamic> map) {
		return OrganizationExpertDTO( 
			fullName: map['FullName'],
			fieldList: map['FieldList'] != null ? map['FieldList'].cast<String>().toList() : null,
			userCode: map['UserCode'],
			userName: map['UserName'],
			headImageUrl: map['HeadImageUrl'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(fullName != null)
			map['FullName'] = fullName;
		if(fieldList != null)
			map['FieldList'] = fieldList;
		return map;
	}
}

class FindOrganizationExpertsRequest extends TokenRequest{
	String? organizationCode;
	String? expertName;

	FindOrganizationExpertsRequest({
		this.organizationCode,
		this.expertName,
		String? token,
	}) : super(
			token: token,
		);

	factory FindOrganizationExpertsRequest.fromJson(Map<String, dynamic> map) {
		return FindOrganizationExpertsRequest( 
			organizationCode: map['OrganizationCode'],
			expertName: map['ExpertName'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		if(expertName != null)
			map['ExpertName'] = expertName;
		return map;
	}
}

enum ConsultationReportMode {
	ExpertReport,
	ApplicantReport,
}

class ApplyConsultationRequest extends TokenRequest{
	String? expertUserCode;
	String? deviceCode;
	String? scanPosition;
	DateTime? consultationTime;
	List<DataItemDTO >? patientDatas;
	String? patientCode;
	ConsultationReportMode reportMode;
	String? diseases;
	String? scanUserCode;
	String? expertOrganizationCode;
	String? applyUserCode;
	String? primaryDiagnosis;

	ApplyConsultationRequest({
		this.expertUserCode,
		this.deviceCode,
		this.scanPosition,
		this.consultationTime,
		this.patientDatas,
		this.patientCode,
		this.reportMode = ConsultationReportMode.ExpertReport,
		this.diseases,
		this.scanUserCode,
		this.expertOrganizationCode,
		this.applyUserCode,
		this.primaryDiagnosis,
		String? token,
	}) : super(
			token: token,
		);

	factory ApplyConsultationRequest.fromJson(Map<String, dynamic> map) {
		return ApplyConsultationRequest( 
			expertUserCode: map['ExpertUserCode'],
			deviceCode: map['DeviceCode'],
			scanPosition: map['ScanPosition'],
			consultationTime: map['ConsultationTime'] != null ? DateTime.parse(map['ConsultationTime']) : null,
			patientDatas: map['PatientDatas'] != null ? (map['PatientDatas'] as List).map((e)=>DataItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			patientCode: map['PatientCode'],
			reportMode: ConsultationReportMode.values.firstWhere((e) => e.index == map['ReportMode']),
			diseases: map['Diseases'],
			scanUserCode: map['ScanUserCode'],
			expertOrganizationCode: map['ExpertOrganizationCode'],
			applyUserCode: map['ApplyUserCode'],
			primaryDiagnosis: map['PrimaryDiagnosis'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(expertUserCode != null)
			map['ExpertUserCode'] = expertUserCode;
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		if(scanPosition != null)
			map['ScanPosition'] = scanPosition;
		if(consultationTime != null)
			map['ConsultationTime'] = JsonRpcUtils.dateFormat(consultationTime!);
		if(patientDatas != null)
			map['PatientDatas'] = patientDatas;
		if(patientCode != null)
			map['PatientCode'] = patientCode;
		map['ReportMode'] = reportMode.index;
		if(diseases != null)
			map['Diseases'] = diseases;
		if(scanUserCode != null)
			map['ScanUserCode'] = scanUserCode;
		if(expertOrganizationCode != null)
			map['ExpertOrganizationCode'] = expertOrganizationCode;
		if(applyUserCode != null)
			map['ApplyUserCode'] = applyUserCode;
		if(primaryDiagnosis != null)
			map['PrimaryDiagnosis'] = primaryDiagnosis;
		return map;
	}
}

class UpdateConsultationRequest extends TokenRequest{
	String? consultationCode;
	String? expertUserCode;
	String? deviceCode;
	String? scanPosition;
	DateTime? consultationTime;
	String? diseases;
	String? scanUserCode;
	String? expertOrganizationCode;
	String? applyUserCode;
	String? primaryDiagnosis;

	UpdateConsultationRequest({
		this.consultationCode,
		this.expertUserCode,
		this.deviceCode,
		this.scanPosition,
		this.consultationTime,
		this.diseases,
		this.scanUserCode,
		this.expertOrganizationCode,
		this.applyUserCode,
		this.primaryDiagnosis,
		String? token,
	}) : super(
			token: token,
		);

	factory UpdateConsultationRequest.fromJson(Map<String, dynamic> map) {
		return UpdateConsultationRequest( 
			consultationCode: map['ConsultationCode'],
			expertUserCode: map['ExpertUserCode'],
			deviceCode: map['DeviceCode'],
			scanPosition: map['ScanPosition'],
			consultationTime: map['ConsultationTime'] != null ? DateTime.parse(map['ConsultationTime']) : null,
			diseases: map['Diseases'],
			scanUserCode: map['ScanUserCode'],
			expertOrganizationCode: map['ExpertOrganizationCode'],
			applyUserCode: map['ApplyUserCode'],
			primaryDiagnosis: map['PrimaryDiagnosis'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		if(expertUserCode != null)
			map['ExpertUserCode'] = expertUserCode;
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		if(scanPosition != null)
			map['ScanPosition'] = scanPosition;
		if(consultationTime != null)
			map['ConsultationTime'] = JsonRpcUtils.dateFormat(consultationTime!);
		if(diseases != null)
			map['Diseases'] = diseases;
		if(scanUserCode != null)
			map['ScanUserCode'] = scanUserCode;
		if(expertOrganizationCode != null)
			map['ExpertOrganizationCode'] = expertOrganizationCode;
		if(applyUserCode != null)
			map['ApplyUserCode'] = applyUserCode;
		if(primaryDiagnosis != null)
			map['PrimaryDiagnosis'] = primaryDiagnosis;
		return map;
	}
}

class FindAssistantExpertsRequest extends TokenRequest{
	String? expertName;

	FindAssistantExpertsRequest({
		this.expertName,
		String? token,
	}) : super(
			token: token,
		);

	factory FindAssistantExpertsRequest.fromJson(Map<String, dynamic> map) {
		return FindAssistantExpertsRequest( 
			expertName: map['ExpertName'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(expertName != null)
			map['ExpertName'] = expertName;
		return map;
	}
}

enum ConsultationStatusEnum {
	placeHolder_0,
	Applied,
	Withdrawn,
	Rejected,
	ToStart,
	InProgress,
	PendingReport,
	End,
}

class ConsultationPageDTO {
	String? consultationCode;
	List<DataItemDTO >? patientDatas;
	ConsultationStatusEnum consultationStatus;
	String? patientName;
	String? sex;
	String? applyOrganizationName;

	ConsultationPageDTO({
		this.consultationCode,
		this.patientDatas,
		this.consultationStatus = ConsultationStatusEnum.Applied,
		this.patientName,
		this.sex,
		this.applyOrganizationName,
	});

	factory ConsultationPageDTO.fromJson(Map<String, dynamic> map) {
		return ConsultationPageDTO( 
			consultationCode: map['ConsultationCode'],
			patientDatas: map['PatientDatas'] != null ? (map['PatientDatas'] as List).map((e)=>DataItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			consultationStatus: ConsultationStatusEnum.values.firstWhere((e) => e.index == map['ConsultationStatus']),
			patientName: map['PatientName'],
			sex: map['Sex'],
			applyOrganizationName: map['ApplyOrganizationName'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		if(patientDatas != null)
			map['PatientDatas'] = patientDatas;
		map['ConsultationStatus'] = consultationStatus.index;
		if(patientName != null)
			map['PatientName'] = patientName;
		if(sex != null)
			map['Sex'] = sex;
		if(applyOrganizationName != null)
			map['ApplyOrganizationName'] = applyOrganizationName;
		return map;
	}
}

enum QueryConsultationStatusEnum {
	All,
	Applied,
	Withdrawn,
	Rejected,
	ToStart,
	InProgress,
	PendingReport,
	End,
}

enum ConsultationQueryTypeEnum {
	MyApply,
	MyArrange,
	MyJoin,
}

class FindConsultationByPageRequest extends PageRequest{
	String? keyword;
	String? expertUserName;
	DateTime? startDate;
	DateTime? endDate;
	QueryConsultationStatusEnum consultationStatus;
	ConsultationQueryTypeEnum consultationQueryType;

	FindConsultationByPageRequest({
		this.keyword,
		this.expertUserName,
		this.startDate,
		this.endDate,
		this.consultationStatus = QueryConsultationStatusEnum.All,
		this.consultationQueryType = ConsultationQueryTypeEnum.MyApply,
		int pageIndex = 0,
		int pageSize = 0,
		String? token,
	}) : super(
			pageIndex: pageIndex,
			pageSize: pageSize,
			token: token,
		);

	factory FindConsultationByPageRequest.fromJson(Map<String, dynamic> map) {
		return FindConsultationByPageRequest( 
			keyword: map['Keyword'],
			expertUserName: map['ExpertUserName'],
			startDate: map['StartDate'] != null ? DateTime.parse(map['StartDate']) : null,
			endDate: map['EndDate'] != null ? DateTime.parse(map['EndDate']) : null,
			consultationStatus: QueryConsultationStatusEnum.values.firstWhere((e) => e.index == map['ConsultationStatus']),
			consultationQueryType: ConsultationQueryTypeEnum.values.firstWhere((e) => e.index == map['ConsultationQueryType']),
			pageIndex: map['PageIndex'],
			pageSize: map['PageSize'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(keyword != null)
			map['Keyword'] = keyword;
		if(expertUserName != null)
			map['ExpertUserName'] = expertUserName;
		if(startDate != null)
			map['StartDate'] = JsonRpcUtils.dateFormat(startDate!);
		if(endDate != null)
			map['EndDate'] = JsonRpcUtils.dateFormat(endDate!);
		map['ConsultationStatus'] = consultationStatus.index;
		map['ConsultationQueryType'] = consultationQueryType.index;
		return map;
	}
}

class ConsultationFileDTO {
	String? sourceUrl;
	String? previewImageUrl;
	String? coverImageUrl;
	DateTime? createTime;
	String? creatorCode;
	RemedicalFileDataTypeEnum fileDataType;

	ConsultationFileDTO({
		this.sourceUrl,
		this.previewImageUrl,
		this.coverImageUrl,
		this.createTime,
		this.creatorCode,
		this.fileDataType = RemedicalFileDataTypeEnum.VinnoVidSingle,
	});

	factory ConsultationFileDTO.fromJson(Map<String, dynamic> map) {
		return ConsultationFileDTO( 
			sourceUrl: map['SourceUrl'],
			previewImageUrl: map['PreviewImageUrl'],
			coverImageUrl: map['CoverImageUrl'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			creatorCode: map['CreatorCode'],
			fileDataType: RemedicalFileDataTypeEnum.values.firstWhere((e) => e.index == map['FileDataType']),
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(sourceUrl != null)
			map['SourceUrl'] = sourceUrl;
		if(previewImageUrl != null)
			map['PreviewImageUrl'] = previewImageUrl;
		if(coverImageUrl != null)
			map['CoverImageUrl'] = coverImageUrl;
		if(createTime != null)
			map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
		if(creatorCode != null)
			map['CreatorCode'] = creatorCode;
		map['FileDataType'] = fileDataType.index;
		return map;
	}
}

enum LiveConsultationMemberStatus {
	Default,
	Accepted,
	Rejected,
	Joined,
	Left,
}

class ConsultationMemberDTO {
	String? memberCode;
	String? memberOrganizationCode;
	String? memberDepartmentCode;
	LiveConsultationMemberStatus memberStatus;
	DateTime? memberOperateTime;

	ConsultationMemberDTO({
		this.memberCode,
		this.memberOrganizationCode,
		this.memberDepartmentCode,
		this.memberStatus = LiveConsultationMemberStatus.Default,
		this.memberOperateTime,
	});

	factory ConsultationMemberDTO.fromJson(Map<String, dynamic> map) {
		return ConsultationMemberDTO( 
			memberCode: map['MemberCode'],
			memberOrganizationCode: map['MemberOrganizationCode'],
			memberDepartmentCode: map['MemberDepartmentCode'],
			memberStatus: LiveConsultationMemberStatus.values.firstWhere((e) => e.index == map['MemberStatus']),
			memberOperateTime: map['MemberOperateTime'] != null ? DateTime.parse(map['MemberOperateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(memberCode != null)
			map['MemberCode'] = memberCode;
		if(memberOrganizationCode != null)
			map['MemberOrganizationCode'] = memberOrganizationCode;
		if(memberDepartmentCode != null)
			map['MemberDepartmentCode'] = memberDepartmentCode;
		map['MemberStatus'] = memberStatus.index;
		if(memberOperateTime != null)
			map['MemberOperateTime'] = JsonRpcUtils.dateFormat(memberOperateTime!);
		return map;
	}
}

enum ConsultationReminderTimeEnum {
	FifteenMinutes,
	ThirtyMinutes,
	SixtyMinutes,
	OneDay,
}

enum ConsultationReminderMode {
	Application,
	SMS,
}

class ConsultationReminderDTO {
	ConsultationReminderTimeEnum consultationReminderTime;
	ConsultationReminderMode consultationReminderMode;

	ConsultationReminderDTO({
		this.consultationReminderTime = ConsultationReminderTimeEnum.FifteenMinutes,
		this.consultationReminderMode = ConsultationReminderMode.Application,
	});

	factory ConsultationReminderDTO.fromJson(Map<String, dynamic> map) {
		return ConsultationReminderDTO( 
			consultationReminderTime: ConsultationReminderTimeEnum.values.firstWhere((e) => e.index == map['ConsultationReminderTime']),
			consultationReminderMode: ConsultationReminderMode.values.firstWhere((e) => e.index == map['ConsultationReminderMode']),
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['ConsultationReminderTime'] = consultationReminderTime.index;
		map['ConsultationReminderMode'] = consultationReminderMode.index;
		return map;
	}
}

class ConsultationDetailDTO {
	String? consultationCode;
	String? applyOrganizationCode;
	String? applyDepartmentCode;
	String? applyUserCode;
	String? expertOrganizationCode;
	String? expertDepartmentCode;
	String? expertUserCode;
	String? deviceCode;
	String? operateUserCode;
	String? scanUserCode;
	String? patientCode;
	OrganizationPatientTypeEnum patientType;
	String? scanPosition;
	DateTime? createTime;
	DateTime? consultationTime;
	DateTime? consultationTimeEnd;
	ConsultationStatusEnum consultationStatus;
	String? applyOrganizationName;
	String? applyUserName;
	String? operateUserName;
	String? scanUserName;
	String? expertOrganizationName;
	String? expertUserName;
	String? deviceName;
	String? patientName;
	String? sex;
	List<DataItemDTO >? patientDatas;
	List<ConsultationFileDTO >? fileInfos;
	String? rejectReason;
	String? location;
	List<ConsultationMemberDTO >? consultationMembers;
	String? description;
	List<ConsultationReminderDTO >? consultationReminders;
	String? approverCode;
	ConsultationReportMode reportMode;
	String? diseases;
	String? primaryDiagnosis;
	String? initiatorCode;

	ConsultationDetailDTO({
		this.consultationCode,
		this.applyOrganizationCode,
		this.applyDepartmentCode,
		this.applyUserCode,
		this.expertOrganizationCode,
		this.expertDepartmentCode,
		this.expertUserCode,
		this.deviceCode,
		this.operateUserCode,
		this.scanUserCode,
		this.patientCode,
		this.patientType = OrganizationPatientTypeEnum.Person,
		this.scanPosition,
		this.createTime,
		this.consultationTime,
		this.consultationTimeEnd,
		this.consultationStatus = ConsultationStatusEnum.Applied,
		this.applyOrganizationName,
		this.applyUserName,
		this.operateUserName,
		this.scanUserName,
		this.expertOrganizationName,
		this.expertUserName,
		this.deviceName,
		this.patientName,
		this.sex,
		this.patientDatas,
		this.fileInfos,
		this.rejectReason,
		this.location,
		this.consultationMembers,
		this.description,
		this.consultationReminders,
		this.approverCode,
		this.reportMode = ConsultationReportMode.ExpertReport,
		this.diseases,
		this.primaryDiagnosis,
		this.initiatorCode,
	});

	factory ConsultationDetailDTO.fromJson(Map<String, dynamic> map) {
		return ConsultationDetailDTO( 
			consultationCode: map['ConsultationCode'],
			applyOrganizationCode: map['ApplyOrganizationCode'],
			applyDepartmentCode: map['ApplyDepartmentCode'],
			applyUserCode: map['ApplyUserCode'],
			expertOrganizationCode: map['ExpertOrganizationCode'],
			expertDepartmentCode: map['ExpertDepartmentCode'],
			expertUserCode: map['ExpertUserCode'],
			deviceCode: map['DeviceCode'],
			operateUserCode: map['OperateUserCode'],
			scanUserCode: map['ScanUserCode'],
			patientCode: map['PatientCode'],
			patientType: OrganizationPatientTypeEnum.values.firstWhere((e) => e.index == map['PatientType']),
			scanPosition: map['ScanPosition'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			consultationTime: map['ConsultationTime'] != null ? DateTime.parse(map['ConsultationTime']) : null,
			consultationTimeEnd: map['ConsultationTimeEnd'] != null ? DateTime.parse(map['ConsultationTimeEnd']) : null,
			consultationStatus: ConsultationStatusEnum.values.firstWhere((e) => e.index == map['ConsultationStatus']),
			applyOrganizationName: map['ApplyOrganizationName'],
			applyUserName: map['ApplyUserName'],
			operateUserName: map['OperateUserName'],
			scanUserName: map['ScanUserName'],
			expertOrganizationName: map['ExpertOrganizationName'],
			expertUserName: map['ExpertUserName'],
			deviceName: map['DeviceName'],
			patientName: map['PatientName'],
			sex: map['Sex'],
			patientDatas: map['PatientDatas'] != null ? (map['PatientDatas'] as List).map((e)=>DataItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			fileInfos: map['FileInfos'] != null ? (map['FileInfos'] as List).map((e)=>ConsultationFileDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			rejectReason: map['RejectReason'],
			location: map['Location'],
			consultationMembers: map['ConsultationMembers'] != null ? (map['ConsultationMembers'] as List).map((e)=>ConsultationMemberDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			description: map['Description'],
			consultationReminders: map['ConsultationReminders'] != null ? (map['ConsultationReminders'] as List).map((e)=>ConsultationReminderDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			approverCode: map['ApproverCode'],
			reportMode: ConsultationReportMode.values.firstWhere((e) => e.index == map['ReportMode']),
			diseases: map['Diseases'],
			primaryDiagnosis: map['PrimaryDiagnosis'],
			initiatorCode: map['InitiatorCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		if(applyOrganizationCode != null)
			map['ApplyOrganizationCode'] = applyOrganizationCode;
		if(applyDepartmentCode != null)
			map['ApplyDepartmentCode'] = applyDepartmentCode;
		if(applyUserCode != null)
			map['ApplyUserCode'] = applyUserCode;
		if(expertOrganizationCode != null)
			map['ExpertOrganizationCode'] = expertOrganizationCode;
		if(expertDepartmentCode != null)
			map['ExpertDepartmentCode'] = expertDepartmentCode;
		if(expertUserCode != null)
			map['ExpertUserCode'] = expertUserCode;
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		if(operateUserCode != null)
			map['OperateUserCode'] = operateUserCode;
		if(scanUserCode != null)
			map['ScanUserCode'] = scanUserCode;
		if(patientCode != null)
			map['PatientCode'] = patientCode;
		map['PatientType'] = patientType.index;
		if(scanPosition != null)
			map['ScanPosition'] = scanPosition;
		if(createTime != null)
			map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
		if(consultationTime != null)
			map['ConsultationTime'] = JsonRpcUtils.dateFormat(consultationTime!);
		if(consultationTimeEnd != null)
			map['ConsultationTimeEnd'] = JsonRpcUtils.dateFormat(consultationTimeEnd!);
		map['ConsultationStatus'] = consultationStatus.index;
		if(applyOrganizationName != null)
			map['ApplyOrganizationName'] = applyOrganizationName;
		if(applyUserName != null)
			map['ApplyUserName'] = applyUserName;
		if(operateUserName != null)
			map['OperateUserName'] = operateUserName;
		if(scanUserName != null)
			map['ScanUserName'] = scanUserName;
		if(expertOrganizationName != null)
			map['ExpertOrganizationName'] = expertOrganizationName;
		if(expertUserName != null)
			map['ExpertUserName'] = expertUserName;
		if(deviceName != null)
			map['DeviceName'] = deviceName;
		if(patientName != null)
			map['PatientName'] = patientName;
		if(sex != null)
			map['Sex'] = sex;
		if(patientDatas != null)
			map['PatientDatas'] = patientDatas;
		if(fileInfos != null)
			map['FileInfos'] = fileInfos;
		if(rejectReason != null)
			map['RejectReason'] = rejectReason;
		if(location != null)
			map['Location'] = location;
		if(consultationMembers != null)
			map['ConsultationMembers'] = consultationMembers;
		if(description != null)
			map['Description'] = description;
		if(consultationReminders != null)
			map['ConsultationReminders'] = consultationReminders;
		if(approverCode != null)
			map['ApproverCode'] = approverCode;
		map['ReportMode'] = reportMode.index;
		if(diseases != null)
			map['Diseases'] = diseases;
		if(primaryDiagnosis != null)
			map['PrimaryDiagnosis'] = primaryDiagnosis;
		if(initiatorCode != null)
			map['InitiatorCode'] = initiatorCode;
		return map;
	}
}

class FindConsultationDetailRequest extends TokenRequest{
	String? consultationCode;

	FindConsultationDetailRequest({
		this.consultationCode,
		String? token,
	}) : super(
			token: token,
		);

	factory FindConsultationDetailRequest.fromJson(Map<String, dynamic> map) {
		return FindConsultationDetailRequest( 
			consultationCode: map['ConsultationCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class ApprovalConsultationRequest extends TokenRequest{
	String? consultationCode;
	String? expertUserCode;
	DateTime? consultationTime;
	DateTime? consultationTimeEnd;
	String? location;
	List<String >? consultationMemberCodes;
	String? description;
	List<ConsultationReminderDTO >? consultationReminders;

	ApprovalConsultationRequest({
		this.consultationCode,
		this.expertUserCode,
		this.consultationTime,
		this.consultationTimeEnd,
		this.location,
		this.consultationMemberCodes,
		this.description,
		this.consultationReminders,
		String? token,
	}) : super(
			token: token,
		);

	factory ApprovalConsultationRequest.fromJson(Map<String, dynamic> map) {
		return ApprovalConsultationRequest( 
			consultationCode: map['ConsultationCode'],
			expertUserCode: map['ExpertUserCode'],
			consultationTime: map['ConsultationTime'] != null ? DateTime.parse(map['ConsultationTime']) : null,
			consultationTimeEnd: map['ConsultationTimeEnd'] != null ? DateTime.parse(map['ConsultationTimeEnd']) : null,
			location: map['Location'],
			consultationMemberCodes: map['ConsultationMemberCodes'] != null ? map['ConsultationMemberCodes'].cast<String>().toList() : null,
			description: map['Description'],
			consultationReminders: map['ConsultationReminders'] != null ? (map['ConsultationReminders'] as List).map((e)=>ConsultationReminderDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		if(expertUserCode != null)
			map['ExpertUserCode'] = expertUserCode;
		if(consultationTime != null)
			map['ConsultationTime'] = JsonRpcUtils.dateFormat(consultationTime!);
		if(consultationTimeEnd != null)
			map['ConsultationTimeEnd'] = JsonRpcUtils.dateFormat(consultationTimeEnd!);
		if(location != null)
			map['Location'] = location;
		if(consultationMemberCodes != null)
			map['ConsultationMemberCodes'] = consultationMemberCodes;
		if(description != null)
			map['Description'] = description;
		if(consultationReminders != null)
			map['ConsultationReminders'] = consultationReminders;
		return map;
	}
}

class RejectApplyConsultationRequest extends TokenRequest{
	String? consultationCode;
	String? reason;

	RejectApplyConsultationRequest({
		this.consultationCode,
		this.reason,
		String? token,
	}) : super(
			token: token,
		);

	factory RejectApplyConsultationRequest.fromJson(Map<String, dynamic> map) {
		return RejectApplyConsultationRequest( 
			consultationCode: map['ConsultationCode'],
			reason: map['Reason'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		if(reason != null)
			map['Reason'] = reason;
		return map;
	}
}

class RevokeConsultationRequest extends TokenRequest{
	String? consultationCode;

	RevokeConsultationRequest({
		this.consultationCode,
		String? token,
	}) : super(
			token: token,
		);

	factory RevokeConsultationRequest.fromJson(Map<String, dynamic> map) {
		return RevokeConsultationRequest( 
			consultationCode: map['ConsultationCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class DeleteConsultationRequest extends TokenRequest{
	String? consultationCode;

	DeleteConsultationRequest({
		this.consultationCode,
		String? token,
	}) : super(
			token: token,
		);

	factory DeleteConsultationRequest.fromJson(Map<String, dynamic> map) {
		return DeleteConsultationRequest( 
			consultationCode: map['ConsultationCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class ConsultationFileBaseDTO {
	String? sourceUrl;
	String? previewImageUrl;
	String? coverImageUrl;
	RemedicalFileDataTypeEnum fileDataType;

	ConsultationFileBaseDTO({
		this.sourceUrl,
		this.previewImageUrl,
		this.coverImageUrl,
		this.fileDataType = RemedicalFileDataTypeEnum.VinnoVidSingle,
	});

	factory ConsultationFileBaseDTO.fromJson(Map<String, dynamic> map) {
		return ConsultationFileBaseDTO( 
			sourceUrl: map['SourceUrl'],
			previewImageUrl: map['PreviewImageUrl'],
			coverImageUrl: map['CoverImageUrl'],
			fileDataType: RemedicalFileDataTypeEnum.values.firstWhere((e) => e.index == map['FileDataType']),
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(sourceUrl != null)
			map['SourceUrl'] = sourceUrl;
		if(previewImageUrl != null)
			map['PreviewImageUrl'] = previewImageUrl;
		if(coverImageUrl != null)
			map['CoverImageUrl'] = coverImageUrl;
		map['FileDataType'] = fileDataType.index;
		return map;
	}
}

class UpdateConsultationFilesInfoRequest extends TokenRequest{
	String? consultationCode;
	List<ConsultationFileBaseDTO >? fileInfos;

	UpdateConsultationFilesInfoRequest({
		this.consultationCode,
		this.fileInfos,
		String? token,
	}) : super(
			token: token,
		);

	factory UpdateConsultationFilesInfoRequest.fromJson(Map<String, dynamic> map) {
		return UpdateConsultationFilesInfoRequest( 
			consultationCode: map['ConsultationCode'],
			fileInfos: map['FileInfos'] != null ? (map['FileInfos'] as List).map((e)=>ConsultationFileBaseDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		if(fileInfos != null)
			map['FileInfos'] = fileInfos;
		return map;
	}
}

class ConsultationAssistantDTO extends UserBaseDTO{
	String? fullName;

	ConsultationAssistantDTO({
		this.fullName,
		String? userCode,
		String? userName,
		String? headImageUrl,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			userCode: userCode,
			userName: userName,
			headImageUrl: headImageUrl,
			createTime: createTime,
			updateTime: updateTime,
		);

	factory ConsultationAssistantDTO.fromJson(Map<String, dynamic> map) {
		return ConsultationAssistantDTO( 
			fullName: map['FullName'],
			userCode: map['UserCode'],
			userName: map['UserName'],
			headImageUrl: map['HeadImageUrl'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(fullName != null)
			map['FullName'] = fullName;
		return map;
	}
}

class FindOrganizationAssistantsRequest extends TokenRequest{
	String? assistantName;

	FindOrganizationAssistantsRequest({
		this.assistantName,
		String? token,
	}) : super(
			token: token,
		);

	factory FindOrganizationAssistantsRequest.fromJson(Map<String, dynamic> map) {
		return FindOrganizationAssistantsRequest( 
			assistantName: map['AssistantName'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(assistantName != null)
			map['AssistantName'] = assistantName;
		return map;
	}
}

class UpdateMyOrganizationAssistantRequest extends TokenRequest{
	String? userCode;

	UpdateMyOrganizationAssistantRequest({
		this.userCode,
		String? token,
	}) : super(
			token: token,
		);

	factory UpdateMyOrganizationAssistantRequest.fromJson(Map<String, dynamic> map) {
		return UpdateMyOrganizationAssistantRequest( 
			userCode: map['UserCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(userCode != null)
			map['UserCode'] = userCode;
		return map;
	}
}

class FindConsultationPatientPageRequest extends PageRequest{
	String? keyword;
	DateTime? startDate;
	DateTime? endDate;

	FindConsultationPatientPageRequest({
		this.keyword,
		this.startDate,
		this.endDate,
		int pageIndex = 0,
		int pageSize = 0,
		String? token,
	}) : super(
			pageIndex: pageIndex,
			pageSize: pageSize,
			token: token,
		);

	factory FindConsultationPatientPageRequest.fromJson(Map<String, dynamic> map) {
		return FindConsultationPatientPageRequest( 
			keyword: map['Keyword'],
			startDate: map['StartDate'] != null ? DateTime.parse(map['StartDate']) : null,
			endDate: map['EndDate'] != null ? DateTime.parse(map['EndDate']) : null,
			pageIndex: map['PageIndex'],
			pageSize: map['PageSize'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(keyword != null)
			map['Keyword'] = keyword;
		if(startDate != null)
			map['StartDate'] = JsonRpcUtils.dateFormat(startDate!);
		if(endDate != null)
			map['EndDate'] = JsonRpcUtils.dateFormat(endDate!);
		return map;
	}
}

enum LiveMemberEnum {
	placeHolder_0,
	User,
	Device,
}

class LiveData {
	int height;
	int width;
	String? rtmpPushUrl;
	String? rtmpPullUrl;
	String? httpPullUrl;
	String? hlsPullUrl;

	LiveData({
		this.height = 0,
		this.width = 0,
		this.rtmpPushUrl,
		this.rtmpPullUrl,
		this.httpPullUrl,
		this.hlsPullUrl,
	});

	factory LiveData.fromJson(Map<String, dynamic> map) {
		return LiveData( 
			height: map['Height'],
			width: map['Width'],
			rtmpPushUrl: map['RtmpPushUrl'],
			rtmpPullUrl: map['RtmpPullUrl'],
			httpPullUrl: map['HttpPullUrl'],
			hlsPullUrl: map['HlsPullUrl'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['Height'] = height;
		map['Width'] = width;
		if(rtmpPushUrl != null)
			map['RtmpPushUrl'] = rtmpPushUrl;
		if(rtmpPullUrl != null)
			map['RtmpPullUrl'] = rtmpPullUrl;
		if(httpPullUrl != null)
			map['HttpPullUrl'] = httpPullUrl;
		if(hlsPullUrl != null)
			map['HlsPullUrl'] = hlsPullUrl;
		return map;
	}
}

class VideoDeviceOutputInfo {
	String? videoDeviceId;
	VideoDeviceSourceTypeEnum videoDeviceSourceType;
	int outputWidth;
	int outputHeight;

	VideoDeviceOutputInfo({
		this.videoDeviceId,
		this.videoDeviceSourceType = VideoDeviceSourceTypeEnum.Desktop,
		this.outputWidth = 0,
		this.outputHeight = 0,
	});

	factory VideoDeviceOutputInfo.fromJson(Map<String, dynamic> map) {
		return VideoDeviceOutputInfo( 
			videoDeviceId: map['VideoDeviceId'],
			videoDeviceSourceType: VideoDeviceSourceTypeEnum.values.firstWhere((e) => e.index == map['VideoDeviceSourceType']),
			outputWidth: map['OutputWidth'],
			outputHeight: map['OutputHeight'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(videoDeviceId != null)
			map['VideoDeviceId'] = videoDeviceId;
		map['VideoDeviceSourceType'] = videoDeviceSourceType.index;
		map['OutputWidth'] = outputWidth;
		map['OutputHeight'] = outputHeight;
		return map;
	}
}

class LiveConsultationMember {
	String? id;
	String? name;
	LiveMemberEnum memberType;
	String? headImageToken;
	bool isOnline;
	bool mute;
	bool videoOpend;
	bool isInitiator;
	bool isBusy;
	LiveConsultationMemberStatus status;
	String? loginServerUrl;
	LoginSource loginSource;
	LiveData? liveData;
	bool mergedChannel;
	int mergedVideoOutputWidth;
	int mergedVideoOutputHeight;
	List<VideoDeviceOutputInfo >? videoDeviceInfos;

	LiveConsultationMember({
		this.id,
		this.name,
		this.memberType = LiveMemberEnum.User,
		this.headImageToken,
		this.isOnline = false,
		this.mute = false,
		this.videoOpend = false,
		this.isInitiator = false,
		this.isBusy = false,
		this.status = LiveConsultationMemberStatus.Default,
		this.loginServerUrl,
		this.loginSource = LoginSource.PC,
		this.liveData,
		this.mergedChannel = false,
		this.mergedVideoOutputWidth = 0,
		this.mergedVideoOutputHeight = 0,
		this.videoDeviceInfos,
	});

	factory LiveConsultationMember.fromJson(Map<String, dynamic> map) {
		return LiveConsultationMember( 
			id: map['Id'],
			name: map['Name'],
			memberType: LiveMemberEnum.values.firstWhere((e) => e.index == map['MemberType']),
			headImageToken: map['HeadImageToken'],
			isOnline: map['IsOnline'],
			mute: map['Mute'],
			videoOpend: map['VideoOpend'],
			isInitiator: map['IsInitiator'],
			isBusy: map['IsBusy'],
			status: LiveConsultationMemberStatus.values.firstWhere((e) => e.index == map['Status']),
			loginServerUrl: map['LoginServerUrl'],
			loginSource: LoginSource.values.firstWhere((e) => e.index == map['LoginSource']),
			liveData: map['LiveData'] != null ? LiveData.fromJson(map['LiveData']) : null,
			mergedChannel: map['MergedChannel'],
			mergedVideoOutputWidth: map['MergedVideoOutputWidth'],
			mergedVideoOutputHeight: map['MergedVideoOutputHeight'],
			videoDeviceInfos: map['VideoDeviceInfos'] != null ? (map['VideoDeviceInfos'] as List).map((e)=>VideoDeviceOutputInfo.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(id != null)
			map['Id'] = id;
		if(name != null)
			map['Name'] = name;
		map['MemberType'] = memberType.index;
		if(headImageToken != null)
			map['HeadImageToken'] = headImageToken;
		map['IsOnline'] = isOnline;
		map['Mute'] = mute;
		map['VideoOpend'] = videoOpend;
		map['IsInitiator'] = isInitiator;
		map['IsBusy'] = isBusy;
		map['Status'] = status.index;
		if(loginServerUrl != null)
			map['LoginServerUrl'] = loginServerUrl;
		map['LoginSource'] = loginSource.index;
		if(liveData != null)
			map['LiveData'] = liveData;
		map['MergedChannel'] = mergedChannel;
		map['MergedVideoOutputWidth'] = mergedVideoOutputWidth;
		map['MergedVideoOutputHeight'] = mergedVideoOutputHeight;
		if(videoDeviceInfos != null)
			map['VideoDeviceInfos'] = videoDeviceInfos;
		return map;
	}
}

class InitiateLiveConsultationResult {
	String? consultationCode;
	String? initiatorCode;
	int roomNo;
	int appId;
	String? userSign;
	List<LiveConsultationMember >? memberLiveDatas;

	InitiateLiveConsultationResult({
		this.consultationCode,
		this.initiatorCode,
		this.roomNo = 0,
		this.appId = 0,
		this.userSign,
		this.memberLiveDatas,
	});

	factory InitiateLiveConsultationResult.fromJson(Map<String, dynamic> map) {
		return InitiateLiveConsultationResult( 
			consultationCode: map['ConsultationCode'],
			initiatorCode: map['InitiatorCode'],
			roomNo: map['RoomNo'],
			appId: map['AppId'],
			userSign: map['UserSign'],
			memberLiveDatas: map['MemberLiveDatas'] != null ? (map['MemberLiveDatas'] as List).map((e)=>LiveConsultationMember.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		if(initiatorCode != null)
			map['InitiatorCode'] = initiatorCode;
		map['RoomNo'] = roomNo;
		map['AppId'] = appId;
		if(userSign != null)
			map['UserSign'] = userSign;
		if(memberLiveDatas != null)
			map['MemberLiveDatas'] = memberLiveDatas;
		return map;
	}
}

class InitiateLiveConsultationRequest extends TokenRequest{
	String? consultationCode;

	InitiateLiveConsultationRequest({
		this.consultationCode,
		String? token,
	}) : super(
			token: token,
		);

	factory InitiateLiveConsultationRequest.fromJson(Map<String, dynamic> map) {
		return InitiateLiveConsultationRequest( 
			consultationCode: map['ConsultationCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class InviteInLiveConsultationResult {
	String? consultationCode;

	InviteInLiveConsultationResult({
		this.consultationCode,
	});

	factory InviteInLiveConsultationResult.fromJson(Map<String, dynamic> map) {
		return InviteInLiveConsultationResult( 
			consultationCode: map['ConsultationCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class InviteInLiveConsultationRequest extends TokenRequest{
	String? consultationCode;
	List<String >? inviteCodes;
	int roomNo;

	InviteInLiveConsultationRequest({
		this.consultationCode,
		this.inviteCodes,
		this.roomNo = 0,
		String? token,
	}) : super(
			token: token,
		);

	factory InviteInLiveConsultationRequest.fromJson(Map<String, dynamic> map) {
		return InviteInLiveConsultationRequest( 
			consultationCode: map['ConsultationCode'],
			inviteCodes: map['InviteCodes'] != null ? map['InviteCodes'].cast<String>().toList() : null,
			roomNo: map['RoomNo'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		if(inviteCodes != null)
			map['InviteCodes'] = inviteCodes;
		map['RoomNo'] = roomNo;
		return map;
	}
}

class CancelInvitingInLiveConsultationResult {
	String? consultationCode;

	CancelInvitingInLiveConsultationResult({
		this.consultationCode,
	});

	factory CancelInvitingInLiveConsultationResult.fromJson(Map<String, dynamic> map) {
		return CancelInvitingInLiveConsultationResult( 
			consultationCode: map['ConsultationCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class CancelInvitingInLiveConsultationRequest extends TokenRequest{
	String? consultationCode;
	List<String >? inviteCodes;

	CancelInvitingInLiveConsultationRequest({
		this.consultationCode,
		this.inviteCodes,
		String? token,
	}) : super(
			token: token,
		);

	factory CancelInvitingInLiveConsultationRequest.fromJson(Map<String, dynamic> map) {
		return CancelInvitingInLiveConsultationRequest( 
			consultationCode: map['ConsultationCode'],
			inviteCodes: map['InviteCodes'] != null ? map['InviteCodes'].cast<String>().toList() : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		if(inviteCodes != null)
			map['InviteCodes'] = inviteCodes;
		return map;
	}
}

class CancelLiveConsultationResult {
	String? consultationCode;

	CancelLiveConsultationResult({
		this.consultationCode,
	});

	factory CancelLiveConsultationResult.fromJson(Map<String, dynamic> map) {
		return CancelLiveConsultationResult( 
			consultationCode: map['ConsultationCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class CancelLiveConsultationRequest extends TokenRequest{
	String? consultationCode;

	CancelLiveConsultationRequest({
		this.consultationCode,
		String? token,
	}) : super(
			token: token,
		);

	factory CancelLiveConsultationRequest.fromJson(Map<String, dynamic> map) {
		return CancelLiveConsultationRequest( 
			consultationCode: map['ConsultationCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class AcceptLiveConsultationResult {
	String? consultationCode;
	String? userCode;
	int roomNo;
	int appId;
	String? userSign;
	List<LiveConsultationMember >? memberLiveDatas;

	AcceptLiveConsultationResult({
		this.consultationCode,
		this.userCode,
		this.roomNo = 0,
		this.appId = 0,
		this.userSign,
		this.memberLiveDatas,
	});

	factory AcceptLiveConsultationResult.fromJson(Map<String, dynamic> map) {
		return AcceptLiveConsultationResult( 
			consultationCode: map['ConsultationCode'],
			userCode: map['UserCode'],
			roomNo: map['RoomNo'],
			appId: map['AppId'],
			userSign: map['UserSign'],
			memberLiveDatas: map['MemberLiveDatas'] != null ? (map['MemberLiveDatas'] as List).map((e)=>LiveConsultationMember.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		if(userCode != null)
			map['UserCode'] = userCode;
		map['RoomNo'] = roomNo;
		map['AppId'] = appId;
		if(userSign != null)
			map['UserSign'] = userSign;
		if(memberLiveDatas != null)
			map['MemberLiveDatas'] = memberLiveDatas;
		return map;
	}
}

class AcceptLiveConsultationRequest extends TokenRequest{
	String? consultationCode;

	AcceptLiveConsultationRequest({
		this.consultationCode,
		String? token,
	}) : super(
			token: token,
		);

	factory AcceptLiveConsultationRequest.fromJson(Map<String, dynamic> map) {
		return AcceptLiveConsultationRequest( 
			consultationCode: map['ConsultationCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class RejectLiveConsultationResult {
	String? consultationCode;

	RejectLiveConsultationResult({
		this.consultationCode,
	});

	factory RejectLiveConsultationResult.fromJson(Map<String, dynamic> map) {
		return RejectLiveConsultationResult( 
			consultationCode: map['ConsultationCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class RejectLiveConsultationRequest extends TokenRequest{
	String? consultationCode;

	RejectLiveConsultationRequest({
		this.consultationCode,
		String? token,
	}) : super(
			token: token,
		);

	factory RejectLiveConsultationRequest.fromJson(Map<String, dynamic> map) {
		return RejectLiveConsultationRequest( 
			consultationCode: map['ConsultationCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class JoinLiveConsultationResult {
	String? consultationCode;

	JoinLiveConsultationResult({
		this.consultationCode,
	});

	factory JoinLiveConsultationResult.fromJson(Map<String, dynamic> map) {
		return JoinLiveConsultationResult( 
			consultationCode: map['ConsultationCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class JoinLiveConsultationRequest extends TokenRequest{
	String? consultationCode;
	String? joinerCode;

	JoinLiveConsultationRequest({
		this.consultationCode,
		this.joinerCode,
		String? token,
	}) : super(
			token: token,
		);

	factory JoinLiveConsultationRequest.fromJson(Map<String, dynamic> map) {
		return JoinLiveConsultationRequest( 
			consultationCode: map['ConsultationCode'],
			joinerCode: map['JoinerCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		if(joinerCode != null)
			map['JoinerCode'] = joinerCode;
		return map;
	}
}

class LeaveLiveConsultationResult {
	String? consultationCode;

	LeaveLiveConsultationResult({
		this.consultationCode,
	});

	factory LeaveLiveConsultationResult.fromJson(Map<String, dynamic> map) {
		return LeaveLiveConsultationResult( 
			consultationCode: map['ConsultationCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class LeaveLiveConsultationRequest extends TokenRequest{
	String? consultationCode;
	String? leaverCode;

	LeaveLiveConsultationRequest({
		this.consultationCode,
		this.leaverCode,
		String? token,
	}) : super(
			token: token,
		);

	factory LeaveLiveConsultationRequest.fromJson(Map<String, dynamic> map) {
		return LeaveLiveConsultationRequest( 
			consultationCode: map['ConsultationCode'],
			leaverCode: map['LeaverCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		if(leaverCode != null)
			map['LeaverCode'] = leaverCode;
		return map;
	}
}

class MuteLiveConsultationResult {
	String? consultationCode;

	MuteLiveConsultationResult({
		this.consultationCode,
	});

	factory MuteLiveConsultationResult.fromJson(Map<String, dynamic> map) {
		return MuteLiveConsultationResult( 
			consultationCode: map['ConsultationCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class MuteLiveConsultationRequest extends TokenRequest{
	String? consultationCode;
	String? memberCode;
	bool mute;

	MuteLiveConsultationRequest({
		this.consultationCode,
		this.memberCode,
		this.mute = false,
		String? token,
	}) : super(
			token: token,
		);

	factory MuteLiveConsultationRequest.fromJson(Map<String, dynamic> map) {
		return MuteLiveConsultationRequest( 
			consultationCode: map['ConsultationCode'],
			memberCode: map['MemberCode'],
			mute: map['Mute'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		if(memberCode != null)
			map['MemberCode'] = memberCode;
		map['Mute'] = mute;
		return map;
	}
}

class SwitchLiveConsultationVideoResult {
	String? consultationCode;

	SwitchLiveConsultationVideoResult({
		this.consultationCode,
	});

	factory SwitchLiveConsultationVideoResult.fromJson(Map<String, dynamic> map) {
		return SwitchLiveConsultationVideoResult( 
			consultationCode: map['ConsultationCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class SwitchLiveConsultationVideoRequest extends TokenRequest{
	String? consultationCode;
	bool opened;

	SwitchLiveConsultationVideoRequest({
		this.consultationCode,
		this.opened = false,
		String? token,
	}) : super(
			token: token,
		);

	factory SwitchLiveConsultationVideoRequest.fromJson(Map<String, dynamic> map) {
		return SwitchLiveConsultationVideoRequest( 
			consultationCode: map['ConsultationCode'],
			opened: map['Opened'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		map['Opened'] = opened;
		return map;
	}
}

class LiveConsultationHeartRateResult {
	String? consultationCode;

	LiveConsultationHeartRateResult({
		this.consultationCode,
	});

	factory LiveConsultationHeartRateResult.fromJson(Map<String, dynamic> map) {
		return LiveConsultationHeartRateResult( 
			consultationCode: map['ConsultationCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class LiveConsultationHeartRateRequest extends TokenRequest{
	String? consultationCode;

	LiveConsultationHeartRateRequest({
		this.consultationCode,
		String? token,
	}) : super(
			token: token,
		);

	factory LiveConsultationHeartRateRequest.fromJson(Map<String, dynamic> map) {
		return LiveConsultationHeartRateRequest( 
			consultationCode: map['ConsultationCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class BoardPointDTO {
	double x;
	double y;

	BoardPointDTO({
		this.x = 0,
		this.y = 0,
	});

	factory BoardPointDTO.fromJson(Map<String, dynamic> map) {
		return BoardPointDTO( 
			x: double.parse(map['X'].toString()),
			y: double.parse(map['Y'].toString()),
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['X'] = x;
		map['Y'] = y;
		return map;
	}
}

enum BoardTypeEnum {
	ReadOnly,
	Ink,
	Line,
	Erase,
}

class SendInteractiveBoardDataRequest extends TokenRequest{
	String? inkColor;
	double inkThickness;
	List<BoardPointDTO >? points;
	BoardTypeEnum boardEditType;
	String? consultationCode;

	SendInteractiveBoardDataRequest({
		this.inkColor,
		this.inkThickness = 0,
		this.points,
		this.boardEditType = BoardTypeEnum.ReadOnly,
		this.consultationCode,
		String? token,
	}) : super(
			token: token,
		);

	factory SendInteractiveBoardDataRequest.fromJson(Map<String, dynamic> map) {
		return SendInteractiveBoardDataRequest( 
			inkColor: map['InkColor'],
			inkThickness: double.parse(map['InkThickness'].toString()),
			points: map['Points'] != null ? (map['Points'] as List).map((e)=>BoardPointDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			boardEditType: BoardTypeEnum.values.firstWhere((e) => e.index == map['BoardEditType']),
			consultationCode: map['ConsultationCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(inkColor != null)
			map['InkColor'] = inkColor;
		map['InkThickness'] = inkThickness;
		if(points != null)
			map['Points'] = points;
		map['BoardEditType'] = boardEditType.index;
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		return map;
	}
}

class AddFollowUpVisitInfoRequest extends TokenRequest{
	String? consultationRecordCode;
	String? patientName;
	String? patientPhone;
	String? generalCase;
	String? clinicalSituation;
	String? doctorCode;
	DateTime? occurredTime;

	AddFollowUpVisitInfoRequest({
		this.consultationRecordCode,
		this.patientName,
		this.patientPhone,
		this.generalCase,
		this.clinicalSituation,
		this.doctorCode,
		this.occurredTime,
		String? token,
	}) : super(
			token: token,
		);

	factory AddFollowUpVisitInfoRequest.fromJson(Map<String, dynamic> map) {
		return AddFollowUpVisitInfoRequest( 
			consultationRecordCode: map['ConsultationRecordCode'],
			patientName: map['PatientName'],
			patientPhone: map['PatientPhone'],
			generalCase: map['GeneralCase'],
			clinicalSituation: map['ClinicalSituation'],
			doctorCode: map['DoctorCode'],
			occurredTime: map['OccurredTime'] != null ? DateTime.parse(map['OccurredTime']) : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationRecordCode != null)
			map['ConsultationRecordCode'] = consultationRecordCode;
		if(patientName != null)
			map['PatientName'] = patientName;
		if(patientPhone != null)
			map['PatientPhone'] = patientPhone;
		if(generalCase != null)
			map['GeneralCase'] = generalCase;
		if(clinicalSituation != null)
			map['ClinicalSituation'] = clinicalSituation;
		if(doctorCode != null)
			map['DoctorCode'] = doctorCode;
		if(occurredTime != null)
			map['OccurredTime'] = JsonRpcUtils.dateFormat(occurredTime!);
		return map;
	}
}

class DeleteFollowUpVisitInfoRequest extends TokenRequest{
	String? followUpVisitCode;

	DeleteFollowUpVisitInfoRequest({
		this.followUpVisitCode,
		String? token,
	}) : super(
			token: token,
		);

	factory DeleteFollowUpVisitInfoRequest.fromJson(Map<String, dynamic> map) {
		return DeleteFollowUpVisitInfoRequest( 
			followUpVisitCode: map['FollowUpVisitCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(followUpVisitCode != null)
			map['FollowUpVisitCode'] = followUpVisitCode;
		return map;
	}
}

class UpdateFollowUpVisitInfoRequest extends TokenRequest{
	String? followUpVisitCode;
	String? generalCase;
	String? clinicalSituation;

	UpdateFollowUpVisitInfoRequest({
		this.followUpVisitCode,
		this.generalCase,
		this.clinicalSituation,
		String? token,
	}) : super(
			token: token,
		);

	factory UpdateFollowUpVisitInfoRequest.fromJson(Map<String, dynamic> map) {
		return UpdateFollowUpVisitInfoRequest( 
			followUpVisitCode: map['FollowUpVisitCode'],
			generalCase: map['GeneralCase'],
			clinicalSituation: map['ClinicalSituation'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(followUpVisitCode != null)
			map['FollowUpVisitCode'] = followUpVisitCode;
		if(generalCase != null)
			map['GeneralCase'] = generalCase;
		if(clinicalSituation != null)
			map['ClinicalSituation'] = clinicalSituation;
		return map;
	}
}

class FollowUpVisitDTO {
	String? followUpVisitCode;
	String? consultationRecordCode;
	String? patientName;
	String? patientPhone;
	String? generalCase;
	String? clinicalSituation;
	String? doctorCode;
	String? doctorName;
	DateTime? occurredTime;

	FollowUpVisitDTO({
		this.followUpVisitCode,
		this.consultationRecordCode,
		this.patientName,
		this.patientPhone,
		this.generalCase,
		this.clinicalSituation,
		this.doctorCode,
		this.doctorName,
		this.occurredTime,
	});

	factory FollowUpVisitDTO.fromJson(Map<String, dynamic> map) {
		return FollowUpVisitDTO( 
			followUpVisitCode: map['FollowUpVisitCode'],
			consultationRecordCode: map['ConsultationRecordCode'],
			patientName: map['PatientName'],
			patientPhone: map['PatientPhone'],
			generalCase: map['GeneralCase'],
			clinicalSituation: map['ClinicalSituation'],
			doctorCode: map['DoctorCode'],
			doctorName: map['DoctorName'],
			occurredTime: map['OccurredTime'] != null ? DateTime.parse(map['OccurredTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(followUpVisitCode != null)
			map['FollowUpVisitCode'] = followUpVisitCode;
		if(consultationRecordCode != null)
			map['ConsultationRecordCode'] = consultationRecordCode;
		if(patientName != null)
			map['PatientName'] = patientName;
		if(patientPhone != null)
			map['PatientPhone'] = patientPhone;
		if(generalCase != null)
			map['GeneralCase'] = generalCase;
		if(clinicalSituation != null)
			map['ClinicalSituation'] = clinicalSituation;
		if(doctorCode != null)
			map['DoctorCode'] = doctorCode;
		if(doctorName != null)
			map['DoctorName'] = doctorName;
		if(occurredTime != null)
			map['OccurredTime'] = JsonRpcUtils.dateFormat(occurredTime!);
		return map;
	}
}

class GetFollowUpVisitInfoRequest extends TokenRequest{
	String? consultationRecordCode;

	GetFollowUpVisitInfoRequest({
		this.consultationRecordCode,
		String? token,
	}) : super(
			token: token,
		);

	factory GetFollowUpVisitInfoRequest.fromJson(Map<String, dynamic> map) {
		return GetFollowUpVisitInfoRequest( 
			consultationRecordCode: map['ConsultationRecordCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationRecordCode != null)
			map['ConsultationRecordCode'] = consultationRecordCode;
		return map;
	}
}

class GetFollowUpVisitInfoDetailRequest extends TokenRequest{
	String? followUpVisitCode;

	GetFollowUpVisitInfoDetailRequest({
		this.followUpVisitCode,
		String? token,
	}) : super(
			token: token,
		);

	factory GetFollowUpVisitInfoDetailRequest.fromJson(Map<String, dynamic> map) {
		return GetFollowUpVisitInfoDetailRequest( 
			followUpVisitCode: map['FollowUpVisitCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(followUpVisitCode != null)
			map['FollowUpVisitCode'] = followUpVisitCode;
		return map;
	}
}

class SetNeedFollowUpVisitRequest extends TokenRequest{
	String? consultationRecordCode;
	bool isNeed;

	SetNeedFollowUpVisitRequest({
		this.consultationRecordCode,
		this.isNeed = false,
		String? token,
	}) : super(
			token: token,
		);

	factory SetNeedFollowUpVisitRequest.fromJson(Map<String, dynamic> map) {
		return SetNeedFollowUpVisitRequest( 
			consultationRecordCode: map['ConsultationRecordCode'],
			isNeed: map['IsNeed'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationRecordCode != null)
			map['ConsultationRecordCode'] = consultationRecordCode;
		map['IsNeed'] = isNeed;
		return map;
	}
}

class AddConsultationEvaluateRequest extends TokenRequest{
	String? consultationRecordCode;
	double evaluateScore;
	String? feedback;

	AddConsultationEvaluateRequest({
		this.consultationRecordCode,
		this.evaluateScore = 0,
		this.feedback,
		String? token,
	}) : super(
			token: token,
		);

	factory AddConsultationEvaluateRequest.fromJson(Map<String, dynamic> map) {
		return AddConsultationEvaluateRequest( 
			consultationRecordCode: map['ConsultationRecordCode'],
			evaluateScore: double.parse(map['EvaluateScore'].toString()),
			feedback: map['Feedback'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationRecordCode != null)
			map['ConsultationRecordCode'] = consultationRecordCode;
		map['EvaluateScore'] = evaluateScore;
		if(feedback != null)
			map['Feedback'] = feedback;
		return map;
	}
}

class DeleteConsultationEvaluateRequest extends TokenRequest{
	String? consultationEvaluateCode;

	DeleteConsultationEvaluateRequest({
		this.consultationEvaluateCode,
		String? token,
	}) : super(
			token: token,
		);

	factory DeleteConsultationEvaluateRequest.fromJson(Map<String, dynamic> map) {
		return DeleteConsultationEvaluateRequest( 
			consultationEvaluateCode: map['ConsultationEvaluateCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationEvaluateCode != null)
			map['ConsultationEvaluateCode'] = consultationEvaluateCode;
		return map;
	}
}

class UpdateConsultationEvaluateRequest extends TokenRequest{
	String? consultationEvaluateCode;
	double evaluateScore;
	String? feedback;

	UpdateConsultationEvaluateRequest({
		this.consultationEvaluateCode,
		this.evaluateScore = 0,
		this.feedback,
		String? token,
	}) : super(
			token: token,
		);

	factory UpdateConsultationEvaluateRequest.fromJson(Map<String, dynamic> map) {
		return UpdateConsultationEvaluateRequest( 
			consultationEvaluateCode: map['ConsultationEvaluateCode'],
			evaluateScore: double.parse(map['EvaluateScore'].toString()),
			feedback: map['Feedback'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationEvaluateCode != null)
			map['ConsultationEvaluateCode'] = consultationEvaluateCode;
		map['EvaluateScore'] = evaluateScore;
		if(feedback != null)
			map['Feedback'] = feedback;
		return map;
	}
}

enum EvaluateGradeEnum {
	UnSet,
	Dissatisfaction,
	General,
	Satisfaction,
}

class ConsultationEvaluateDTO {
	String? consultationEvaluateCode;
	String? consultationRecordCode;
	EvaluateGradeEnum evaluateGrade;
	double evaluateScore;
	String? feedback;

	ConsultationEvaluateDTO({
		this.consultationEvaluateCode,
		this.consultationRecordCode,
		this.evaluateGrade = EvaluateGradeEnum.UnSet,
		this.evaluateScore = 0,
		this.feedback,
	});

	factory ConsultationEvaluateDTO.fromJson(Map<String, dynamic> map) {
		return ConsultationEvaluateDTO( 
			consultationEvaluateCode: map['ConsultationEvaluateCode'],
			consultationRecordCode: map['ConsultationRecordCode'],
			evaluateGrade: EvaluateGradeEnum.values.firstWhere((e) => e.index == map['EvaluateGrade']),
			evaluateScore: double.parse(map['EvaluateScore'].toString()),
			feedback: map['Feedback'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(consultationEvaluateCode != null)
			map['ConsultationEvaluateCode'] = consultationEvaluateCode;
		if(consultationRecordCode != null)
			map['ConsultationRecordCode'] = consultationRecordCode;
		map['EvaluateGrade'] = evaluateGrade.index;
		map['EvaluateScore'] = evaluateScore;
		if(feedback != null)
			map['Feedback'] = feedback;
		return map;
	}
}

class GetConsultationEvaluateRequest extends TokenRequest{
	String? consultationRecordCode;

	GetConsultationEvaluateRequest({
		this.consultationRecordCode,
		String? token,
	}) : super(
			token: token,
		);

	factory GetConsultationEvaluateRequest.fromJson(Map<String, dynamic> map) {
		return GetConsultationEvaluateRequest( 
			consultationRecordCode: map['ConsultationRecordCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationRecordCode != null)
			map['ConsultationRecordCode'] = consultationRecordCode;
		return map;
	}
}

class AcceptInvitationRequest extends TokenRequest{
	String? consultationRecordCode;
	bool isAgree;
	String? refusalCause;

	AcceptInvitationRequest({
		this.consultationRecordCode,
		this.isAgree = false,
		this.refusalCause,
		String? token,
	}) : super(
			token: token,
		);

	factory AcceptInvitationRequest.fromJson(Map<String, dynamic> map) {
		return AcceptInvitationRequest( 
			consultationRecordCode: map['ConsultationRecordCode'],
			isAgree: map['IsAgree'],
			refusalCause: map['RefusalCause'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationRecordCode != null)
			map['ConsultationRecordCode'] = consultationRecordCode;
		map['IsAgree'] = isAgree;
		if(refusalCause != null)
			map['RefusalCause'] = refusalCause;
		return map;
	}
}

enum SyncTypeEnum {
	InitiateLiveConsultation,
	AcceptLiveConsultation,
	RejectLiveConsultation,
	CancelInitiatorLiveConsultation,
	LeaveLiveConsultation,
	CloseLiveConsultation,
	MuteLiveConsultation,
	SwitchLiveConsultationVideo,
	JoinLiveConsultation,
	NewworkErrLiveConsultation,
	CancelInvitingInLiveConsultation,
	InviteInLiveConsultation,
}

enum MongoDBActionTypeEnum {
	InsertOne,
	InsertOneAsync,
	InsertMany,
	InsertManyAsync,
	DeleteOne,
	DeleteOneAsync,
	DeleteMany,
	DeleteManyAsync,
	FindOneAndDelete,
	FindOneAndDeleteAsync,
	ReplaceOne,
	ReplaceOneAsync,
	FindOneAndReplace,
	FindOneAndReplaceAsync,
	UpdateOne,
	UpdateOneAsync,
	UpdateMany,
	UpdateManyAsync,
	FindOneAndUpdate,
	FindOneAndUpdateAsync,
}

class OperationLogDTO {
	int id;
	String? collectionName;
	MongoDBActionTypeEnum actionType;
	String? bsonContent;
	String? filterContent;
	DateTime? createTime;
	String? code;
	String? sourceUrl;

	OperationLogDTO({
		this.id = 0,
		this.collectionName,
		this.actionType = MongoDBActionTypeEnum.InsertOne,
		this.bsonContent,
		this.filterContent,
		this.createTime,
		this.code,
		this.sourceUrl,
	});

	factory OperationLogDTO.fromJson(Map<String, dynamic> map) {
		return OperationLogDTO( 
			id: map['Id'],
			collectionName: map['CollectionName'],
			actionType: MongoDBActionTypeEnum.values.firstWhere((e) => e.index == map['ActionType']),
			bsonContent: map['BsonContent'],
			filterContent: map['FilterContent'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			code: map['Code'],
			sourceUrl: map['SourceUrl'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['Id'] = id;
		if(collectionName != null)
			map['CollectionName'] = collectionName;
		map['ActionType'] = actionType.index;
		if(bsonContent != null)
			map['BsonContent'] = bsonContent;
		if(filterContent != null)
			map['FilterContent'] = filterContent;
		if(createTime != null)
			map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
		if(code != null)
			map['Code'] = code;
		if(sourceUrl != null)
			map['SourceUrl'] = sourceUrl;
		return map;
	}
}

class SyncReceiveServiceDataRequest {
	SyncTypeEnum syncType;
	String? serviceDataJson;
	List<OperationLogDTO >? oplogs;
	String? sourceUrl;
	String? serverID;

	SyncReceiveServiceDataRequest({
		this.syncType = SyncTypeEnum.InitiateLiveConsultation,
		this.serviceDataJson,
		this.oplogs,
		this.sourceUrl,
		this.serverID,
	});

	factory SyncReceiveServiceDataRequest.fromJson(Map<String, dynamic> map) {
		return SyncReceiveServiceDataRequest( 
			syncType: SyncTypeEnum.values.firstWhere((e) => e.index == map['SyncType']),
			serviceDataJson: map['ServiceDataJson'],
			oplogs: map['Oplogs'] != null ? (map['Oplogs'] as List).map((e)=>OperationLogDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			sourceUrl: map['SourceUrl'],
			serverID: map['ServerID'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['SyncType'] = syncType.index;
		if(serviceDataJson != null)
			map['ServiceDataJson'] = serviceDataJson;
		if(oplogs != null)
			map['Oplogs'] = oplogs;
		if(sourceUrl != null)
			map['SourceUrl'] = sourceUrl;
		if(serverID != null)
			map['ServerID'] = serverID;
		return map;
	}
}


