enum MessageTypeEnum {
	placeHolder_0,
	Connection,
	Disconnect,
	Notify,
	KeepNotify,
	Ask,
	Ack,
}

enum NotificationTypeEnum {
	placeHolder_0,
	Message,
	Action,
	Update,
	NotifyRecordCodesMessage,
	FinishNotifyRecordsMessage,
}

class NotifyMessage {
	int length;
	MessageTypeEnum messageType;
	NotificationTypeEnum notificationType;
	String? message;

	NotifyMessage({
		this.length = 0,
		this.messageType = MessageTypeEnum.Connection,
		this.notificationType = NotificationTypeEnum.Message,
		this.message,
	});

	factory NotifyMessage.fromJson(Map<String, dynamic> map) {
		return NotifyMessage( 
			length: map['Length'],
			messageType: MessageTypeEnum.values.firstWhere((e) => e.index == map['MessageType']),
			notificationType: NotificationTypeEnum.values.firstWhere((e) => e.index == map['NotificationType']),
			message: map['Message'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['Length'] = length;
		map['MessageType'] = messageType.index;
		map['NotificationType'] = notificationType.index;
		if(message != null)
			map['Message'] = message;
		return map;
	}
}

class SendNotificationRequest {
	String? clientId;
	NotifyMessage? message;

	SendNotificationRequest({
		this.clientId,
		this.message,
	});

	factory SendNotificationRequest.fromJson(Map<String, dynamic> map) {
		return SendNotificationRequest( 
			clientId: map['ClientId'],
			message: map['Message'] != null ? NotifyMessage.fromJson(map['Message']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(clientId != null)
			map['ClientId'] = clientId;
		if(message != null)
			map['Message'] = message;
		return map;
	}
}

class BroadcastNotificationRequest {
	List<String >? clientIds;
	NotifyMessage? message;
	String? msgQueueId;

	BroadcastNotificationRequest({
		this.clientIds,
		this.message,
		this.msgQueueId,
	});

	factory BroadcastNotificationRequest.fromJson(Map<String, dynamic> map) {
		return BroadcastNotificationRequest( 
			clientIds: map['ClientIds'] != null ? map['ClientIds'].cast<String>().toList() : null,
			message: map['Message'] != null ? NotifyMessage.fromJson(map['Message']) : null,
			msgQueueId: map['MsgQueueId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(clientIds != null)
			map['ClientIds'] = clientIds;
		if(message != null)
			map['Message'] = message;
		if(msgQueueId != null)
			map['MsgQueueId'] = msgQueueId;
		return map;
	}
}

class OpenNotifyQueueRequest {
	String? module;

	OpenNotifyQueueRequest({
		this.module,
	});

	factory OpenNotifyQueueRequest.fromJson(Map<String, dynamic> map) {
		return OpenNotifyQueueRequest( 
			module: map['Module'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(module != null)
			map['Module'] = module;
		return map;
	}
}

class CloseNotifyQueueRequest {
	String? msgQueueId;

	CloseNotifyQueueRequest({
		this.msgQueueId,
	});

	factory CloseNotifyQueueRequest.fromJson(Map<String, dynamic> map) {
		return CloseNotifyQueueRequest( 
			msgQueueId: map['MsgQueueId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(msgQueueId != null)
			map['MsgQueueId'] = msgQueueId;
		return map;
	}
}


